@@ -7,9 +7,9 @@
 ####################################################################################################################################
 
 #' EEG data of 160 subjects
-#' 
+#'
 #' A dataset containing EEG data (Staffen et al., 2014) of 160 subjects, 4 variables are measured at ten different locations.
-#' 
+#'
 #' The columns are as follows:
 #' \itemize{
 #'   \item group. Diagnostic group of the subject: Alzheimer's Disease (AD), Mild Cognitive Impairment (MCI), Subject Cognitive Complaints (SCC+, SCC-).
@@ -17,10 +17,10 @@
 #'   \item sex. Sex of the subject: Male (M) or Female (W).
 #'   \item subject. A unique identification of a subject.
 #'   \item variable. The variales measured are activity, complexity, mobility and brain rate coded from 1 to 4.
-#'   \item region. Frontal left/right, central left/right, temporal left/right, occipital left/right, parietal left/right coded as 1 to 10. 
+#'   \item region. Frontal left/right, central left/right, temporal left/right, occipital left/right, parietal left/right coded as 1 to 10.
 #'   \item dimension. Mixing variable and region together, levels range from 1 to 40.
 #' }
-#' 
+#'
 #' @docType data
 #' @keywords datasets
 #' @name EEG
@@ -30,17 +30,17 @@
 
 
 #' Test for no main treatment effect, no main time effect, no simple treatment effect and no interaction between treatment and time
-#' 
+#'
 #' @param data A list containing the data matrices of all groups. The rows are the independent subjects, these observations are assumed to be multivariate normally distributed. The columsn of all matrices need to be in the same order.
 #' @param alpha alpha level used for the test
 #' @return Returns a data frame consisting of the degrees of freedom, the test value, the critical value and the p-value
 #' @keywords internal
 hrm.test.matrices <- function(data, alpha=0.05){
-  
+
   if(!is.list(data)){
     stop("data needs to be a list containing the data matrices of all groups")
   }
-  
+
   a <- length(data)
   n <- rep(0,a)
   d <- rep(0,a)
@@ -52,27 +52,27 @@ hrm.test.matrices <- function(data, alpha=0.05){
     n[i] <- dim(tmp)[1]
     d[i] <- dim(tmp)[2]
   }
-  
+
   if(mean(d) != d[1]){
     stop("The number of measurements for each group need to be the same.")
   }
   d <- d[1]
- 
+
   if(d < 2 & a < 2) {
     stop("At least two measurements per subject or two groups are needed.")
   }
-  
-  
+
+
   # choose appropriate tests based on the number of factors present
   testing <- rep(0, 5)
-  
+
   if(a == 1 & d > 1) { testing <- c(0,0,1,0,0) }
   if(a > 1 & d == 1) { testing <- c(1,1,0,0,0) }
   if(a > 1 & d > 1) { testing <- c(1,1,1,1,1) }
-  
+
   temp0 <- if(testing[1]) { hrm.A.weighted(n,a,d,data,alpha) }
   temp1 <- if(testing[2]) { hrm.A.unweighted(n,a,d,data,alpha) }
-  temp2 <- if(testing[3]) { hrm.B(n,a,d,data,alpha) } 
+  temp2 <- if(testing[3]) { hrm.B(n,a,d,data,alpha) }
   temp3 <- if(testing[4]) { hrm.AB(n,a,d,data,alpha) }
   temp4 <- if(testing[5]) { hrm.A_B(n,a,d,data,alpha) }
 
@@ -85,12 +85,12 @@ hrm.test.matrices <- function(data, alpha=0.05){
   output$data <- data
   output$nonparametric <- FALSE
   class(output) <- "HRM"
-  
+
   return (output)
 }
 
 #' Test for no main effects and interactino effects of one between-subject factor and one crossed within-subject factors
-#' 
+#'
 #' @param X dataframe containing the data in the long table format
 #' @param alpha alpha level used for the test
 #' @param group column name of the data frame X specifying the groups
@@ -103,29 +103,32 @@ hrm.test.matrices <- function(data, alpha=0.05){
 #' @return Returns a data frame consisting of the degrees of freedom, the test value, the critical value and the p-value
 #' @keywords internal
 hrm.test.2.one <- function(X, alpha, group , factor1, subject, data, testing = rep(1,4), formula, nonparametric ){
-  
-  ranked <- NULL
 
-  temp0 <- if(testing[1]) {hrm.1w.1f(X, alpha, group , factor1,  subject, data, "A", paste(as.character(group), " (weighted)"), nonparametric, ranked)}
-  temp1 <- if(testing[2]) {hrm.1w.1f(X, alpha, group , factor1,  subject, data, "Au", paste(as.character(group)), nonparametric, ranked)}
-  temp2 <- if(testing[3]) {hrm.1w.1f(X, alpha, group , factor1,  subject, data, "B", paste(as.character(factor1)), nonparametric, ranked)}
-  temp3 <- if(testing[4]) {hrm.1w.1f(X, alpha, group , factor1, subject, data, "AB", paste(as.character(group), ":",as.character(factor1)), nonparametric, ranked)}
-  
+  ranked <- NULL
+  varQGlobal <- NULL
+
+  temp0 <- if(testing[1]) {hrm.1w.1f(X, alpha, group , factor1,  subject, data, "A", paste(as.character(group), " (weighted)"), nonparametric, ranked, varQGlobal)}
+  temp1 <- if(testing[2]) {hrm.1w.1f(X, alpha, group , factor1,  subject, data, "Au", paste(as.character(group)), nonparametric, ranked, varQGlobal)}
+  temp2 <- if(testing[3]) {hrm.1w.1f(X, alpha, group , factor1,  subject, data, "B", paste(as.character(factor1)), nonparametric, ranked, varQGlobal)}
+  temp3 <- if(testing[4]) {hrm.1w.1f(X, alpha, group , factor1, subject, data, "AB", paste(as.character(group), ":",as.character(factor1)), nonparametric, ranked, varQGlobal)}
+  temp4 <- if(testing[3] & (testing[1] | testing[2])) {hrm.1w.1f(X, alpha, group , factor1, subject, data, "A|B", paste(as.character(group), "|",as.character(factor1)), nonparametric, ranked, varQGlobal)}
+
   output <- list()
-  output$result <- rbind(temp0, temp1, temp2, temp3)
+  output$result <- rbind(temp0, temp1, temp2, temp3,temp4)
   output$formula <- formula
   output$alpha <- alpha
   output$subject <- subject
   output$factors <- list(c(group), c(factor1))
   output$data <- X
+  output$var <- varQGlobal
   output$nonparametric <- nonparametric
   class(output) <- "HRM"
-  
+  rownames(output$result) <- 1:dim(output$result)[1]
   return (output)
 }
 
 #' Test for no main effects and interactino effects of one between-subject factor and two crossed within-subject factors
-#' 
+#'
 #' @param X dataframe containing the data in the long table format
 #' @param alpha alpha level used for the test
 #' @param group column name of the data frame X specifying the groups
@@ -138,17 +141,18 @@ hrm.test.2.one <- function(X, alpha, group , factor1, subject, data, testing = r
 #' @return Returns a data frame consisting of the degrees of freedom, the test value, the critical value and the p-value
 #' @keywords internal
 hrm.test.2.within <- function(X, alpha, group , factor1, factor2, subject, data, testing = rep(1,7), formula, nonparametric ){
-  
+
   ranked <- NULL
+  varQGlobal <- NULL
 
   # create list for storing results; NULL used, because it is ignored by rbind
   temp <- list(NULL, NULL, NULL, NULL, NULL, NULL, NULL)
   for(i in 1:7){
     if(testing[i]) {
-      temp[[i]] <- hrm.1w.2f(X, alpha, group , factor1, factor2, subject, data, H = i, "", nonparametric, ranked )
+      temp[[i]] <- hrm.1w.2f(X, alpha, group , factor1, factor2, subject, data, H = i, "", nonparametric, ranked, varQGlobal )
     }
   }
-  
+
   output <- list()
   output$result <- rbind(temp[[1]], temp[[2]], temp[[3]], temp[[4]], temp[[5]], temp[[6]], temp[[7]])
   output$formula <- formula
@@ -156,16 +160,17 @@ hrm.test.2.within <- function(X, alpha, group , factor1, factor2, subject, data,
   output$subject <- subject
   output$factors <- list(c(group), c(factor1, factor2))
   output$data <- X
+  output$var <- varQGlobal
   output$nonparametric <- nonparametric
   class(output) <- "HRM"
-  
+
   return (output)
 }
 
 
 
 #' Test for no main effects and interaction effects of two crossed between-subject factors and one within-subject factor
-#' 
+#'
 #' @param X dataframe containing the data in the long table format
 #' @param alpha alpha level used for the test
 #' @param group column name of the data frame X specifying the groups
@@ -180,15 +185,16 @@ hrm.test.2.within <- function(X, alpha, group , factor1, factor2, subject, data,
 hrm.test.2.between <- function(X, alpha, group , subgroup, factor, subject, data, testing = rep(1,7), formula, nonparametric ){
 
   ranked <- NULL
-  
+  varQGlobal <- NULL
+
   # create list for storing results; NULL used, because it is ignored by rbind
   temp <- list(NULL, NULL, NULL, NULL, NULL, NULL, NULL)
   for(i in 1:7){
     if(testing[i]) {
-      temp[[i]] <- hrm.2w.1f(X, alpha, group , subgroup, factor, subject, data, H = i, "", nonparametric, ranked )
+      temp[[i]] <- hrm.2w.1f(X, alpha, group , subgroup, factor, subject, data, H = i, "", nonparametric, ranked, varQGlobal )
     }
   }
-  
+
   output <- list()
   output$result <- rbind(temp[[1]], temp[[2]], temp[[3]], temp[[4]], temp[[5]], temp[[6]], temp[[7]])
   output$formula <- formula
@@ -196,15 +202,16 @@ hrm.test.2.between <- function(X, alpha, group , subgroup, factor, subject, data
   output$subject <- subject
   output$factors <- list(c(group, subgroup), c(factor))
   output$data <- X
+  output$var <- varQGlobal
   output$nonparametric <- nonparametric
   class(output) <- "HRM"
- 
+
   return (output)
 }
 
 
 #' Test for no main effects and interaction effects of two crossed between-subject factors and one within-subject factor
-#' 
+#'
 #' @param X dataframe containing the data in the long table format
 #' @param alpha alpha level used for the test
 #' @param group column name of the data frame X specifying the groups
@@ -218,14 +225,15 @@ hrm.test.2.between <- function(X, alpha, group , subgroup, factor, subject, data
 #' @return Returns a data frame consisting of the degrees of freedom, the test value, the critical value and the p-value
 #' @keywords internal
 hrm.test.2.between.within <- function(X, alpha, group , subgroup, factor1, factor2, subject, data, testing = rep(1,15), formula, nonparametric ){
-  
+
   ranked <- NULL
-  
+  varQGlobal <- NULL
+
   # create list for storing results; NULL used, because it is ignored by rbind
   temp <- list(NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL)
   for(i in 1:15){
     if(testing[i]) {
-      temp[[i]] <- hrm.2w.2f(X, alpha, group , subgroup, factor1, factor2, subject, data, H = i, "", nonparametric, ranked )
+      temp[[i]] <- hrm.2w.2f(X, alpha, group , subgroup, factor1, factor2, subject, data, H = i, "", nonparametric, ranked, varQGlobal )
     }
   }
   output <- list()
@@ -235,6 +243,7 @@ hrm.test.2.between.within <- function(X, alpha, group , subgroup, factor1, facto
   output$subject <- subject
   output$factors <- list(c(group, subgroup), c(factor1, factor2))
   output$data <- X
+  output$var <- varQGlobal
   output$nonparametric <- nonparametric
   class(output) <- "HRM"
   return (output)
@@ -242,7 +251,7 @@ hrm.test.2.between.within <- function(X, alpha, group , subgroup, factor1, facto
 
 
 #' Test for no main effects and interaction effects of two crossed between-subject factors and one within-subject factor
-#' 
+#'
 #' @param X dataframe containing the data in the long table format
 #' @param alpha alpha level used for the test
 #' @param group column name of the data frame X specifying the groups
@@ -256,25 +265,26 @@ hrm.test.2.between.within <- function(X, alpha, group , subgroup, factor1, facto
 #' @return Returns a data frame consisting of the degrees of freedom, the test value, the critical value and the p-value
 #' @keywords internal
 hrm.test.3.between <- function(X, alpha, group , factor1, factor2, factor3, subject, data, testing = rep(1,15), formula, nonparametric ){
-  
+
   ranked <- NULL
-  
-  temp0 <- if(testing[1]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3, subject, data, "P", "J", "J", "J",  paste(as.character(group) ), nonparametric, ranked )}
-  temp1 <- if(testing[2]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "P", "J", "J", paste(as.character(factor1)), nonparametric, ranked )}
-  temp2 <- if(testing[3]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "J", "P", "J", paste(as.character(factor2)), nonparametric, ranked )}
-  temp3 <- if(testing[4]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "J", "J", "P", paste(as.character(factor3)), nonparametric, ranked )}
-  temp4 <- if(testing[5]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "P", "J", "J", paste(as.character(group),":",as.character(factor1)), nonparametric, ranked )} 
-  temp5 <- if(testing[6]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "J", "P", "J", paste(as.character(group),":",as.character(factor2)), nonparametric, ranked )}
-  temp6 <- if(testing[7]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "J", "J", "P", paste(as.character(group),":",as.character(factor3)), nonparametric, ranked )}
-  temp7 <- if(testing[8]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "P", "P", "J", paste(as.character(factor1),":",as.character(factor2)), nonparametric, ranked )}
-  temp8 <- if(testing[9]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "P", "J", "P", paste(as.character(factor1),":",as.character(factor3)), nonparametric, ranked )}
-  temp9 <- if(testing[10]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "J", "P", "P",paste(as.character(factor2),":",as.character(factor3)), nonparametric, ranked )}
-  temp10 <- if(testing[11]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "P", "P", "J", paste(as.character(group),":",as.character(factor1), ":", as.character(factor2)), nonparametric, ranked )}
-  temp11 <- if(testing[12]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "P", "J", "P", paste(as.character(group),":",as.character(factor1), ":", as.character(factor3)), nonparametric, ranked )}
-  temp12 <- if(testing[13]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "J", "P", "P", paste(as.character(group),":",as.character(factor2), ":", as.character(factor3)), nonparametric, ranked )}
-  temp13 <- if(testing[14]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "P", "P", "P", paste(as.character(factor1),":",as.character(factor2), ":", as.character(factor3)), nonparametric, ranked )}
-  temp14 <- if(testing[15]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "P", "P", "P", paste(as.character(group),":",as.character(factor1), ":", as.character(factor2), ":", as.character(factor3)), nonparametric, ranked )}
-  
+  varQGlobal <- NULL
+
+  temp0 <- if(testing[1]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3, subject, data, "P", "J", "J", "J",  paste(as.character(group) ), nonparametric, ranked,varQGlobal )}
+  temp1 <- if(testing[2]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "P", "J", "J", paste(as.character(factor1)), nonparametric, ranked,varQGlobal )}
+  temp2 <- if(testing[3]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "J", "P", "J", paste(as.character(factor2)), nonparametric, ranked,varQGlobal )}
+  temp3 <- if(testing[4]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "J", "J", "P", paste(as.character(factor3)), nonparametric, ranked,varQGlobal )}
+  temp4 <- if(testing[5]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "P", "J", "J", paste(as.character(group),":",as.character(factor1)), nonparametric, ranked,varQGlobal )}
+  temp5 <- if(testing[6]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "J", "P", "J", paste(as.character(group),":",as.character(factor2)), nonparametric, ranked,varQGlobal )}
+  temp6 <- if(testing[7]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "J", "J", "P", paste(as.character(group),":",as.character(factor3)), nonparametric, ranked,varQGlobal )}
+  temp7 <- if(testing[8]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "P", "P", "J", paste(as.character(factor1),":",as.character(factor2)), nonparametric, ranked,varQGlobal )}
+  temp8 <- if(testing[9]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "P", "J", "P", paste(as.character(factor1),":",as.character(factor3)), nonparametric, ranked,varQGlobal )}
+  temp9 <- if(testing[10]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "J", "P", "P",paste(as.character(factor2),":",as.character(factor3)), nonparametric, ranked,varQGlobal )}
+  temp10 <- if(testing[11]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "P", "P", "J", paste(as.character(group),":",as.character(factor1), ":", as.character(factor2)), nonparametric, ranked,varQGlobal )}
+  temp11 <- if(testing[12]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "P", "J", "P", paste(as.character(group),":",as.character(factor1), ":", as.character(factor3)), nonparametric, ranked,varQGlobal )}
+  temp12 <- if(testing[13]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "J", "P", "P", paste(as.character(group),":",as.character(factor2), ":", as.character(factor3)), nonparametric, ranked,varQGlobal )}
+  temp13 <- if(testing[14]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "P", "P", "P", paste(as.character(factor1),":",as.character(factor2), ":", as.character(factor3)), nonparametric, ranked,varQGlobal )}
+  temp14 <- if(testing[15]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "P", "P", "P", paste(as.character(group),":",as.character(factor1), ":", as.character(factor2), ":", as.character(factor3)), nonparametric, ranked,varQGlobal )}
+
   output <- list()
   output$result <- rbind(temp0, temp1, temp2, temp3, temp4, temp5, temp6, temp7, temp8, temp9, temp10, temp11, temp12, temp13, temp14)
   output$formula <- formula
@@ -282,14 +292,15 @@ hrm.test.3.between <- function(X, alpha, group , factor1, factor2, factor3, subj
   output$subject <- subject
   output$factors <- list(c(group), c(factor1, factor2, factor3))
   output$data <- X
+  output$var <- varQGlobal
   output$nonparametric <- nonparametric
   class(output) <- "HRM"
   return (output)
 }
 
 
-#' Test for Multi-Factor High-Dimensional Repeated Measures 
-#' 
+#' Test for Multi-Factor High-Dimensional Repeated Measures
+#'
 #' @description Performing main and interaction effects of up to three whole- or subplot-factors. In total, a maximum of four factors can be used. There are two different S3 methods available. The first method requires a list of matrices in the wide table format. The second methodl requres a data.frame in the long table format.
 #' @param data Either a data.frame (one observation per row) or a list with matrices (one subject per row) for all groups containing the data
 #' @param formula A model formula object. The left hand side contains the response variable and the right hand side contains the whole- and subplot factors.
@@ -319,9 +330,9 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject, nonparametr
       stop("alpha level needs to be a number between 0 and 1")
     }
   }
-  
+
   # convert whole/subplot factor columns to type factor
-  tryCatch({ 
+  tryCatch({
     nfactors <- length(attr(terms.formula(formula), "variables"))
     for(i in 3:nfactors){
       cname <- as.character(attr(terms.formula(formula), "variables")[[i]])
@@ -330,16 +341,16 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject, nonparametr
       }
     }
   }, warning = function(w) "", error = function(e) { paste("One of the factor columns could not be converted to a factor variable." ) } )
-  
+
   dat <- model.frame(formula, data)
   dat2 <- data.frame(dat,subj=data[,subject])
 
   m <- ncol(dat)
-  
+
   if(!is.numeric(dat[,1])){
     stop("Response variable needs to be numeric!")
   }
-  
+
   # find out, in which columns are the wholeplot or subplot factors
   s1<-subset(dat2, dat2$subj==dat2$subj[1])
   measurements <- dim(s1)[1]
@@ -360,8 +371,8 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject, nonparametr
   }
   wholeplot <- which(wholeplot==1)
   subplot <- which( subplot==1)
-  
-  
+
+
   if(!(measurements == countSubplotFactor)){
     stop(paste("The number of repeated measurements per subject (", measurements, ") is uneqal to the number of levels of the subplot factors (", countSubplotFactor, ")."))
   }
@@ -380,31 +391,31 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject, nonparametr
   if(length(subplot)>5 & length(wholeplot)<1){
     stop("The model supports up to five subplot-factor when using no wholeplot-factors.")
   }
-  
+
   # Case: no wholeplot, one subpot factor
   if(length(wholeplot) < 1 & length(subplot) == 1){
     factor1 <- colnames(dat2)[subplot[1]]
     x<-attributes(terms.formula(formula))$term.labels
-    
+
     subplot<-colnames(dat2[,subplot])
     X<-data
     data <- colnames(dat)[1]
-    return(hrm.test.1.one(X, alpha , factor1, subject, data, formula, nonparametric ))   
-    
+    return(hrm.test.1.one(X, alpha , factor1, subject, data, formula, nonparametric ))
+
   }
-  
+
   # Case: no wholeplot, two subpot factor
   if(length(wholeplot) < 1 & length(subplot) == 2){
     factor1 <- colnames(dat2)[subplot[1]]
     factor2 <- colnames(dat2)[subplot[2]]
     x<-attributes(terms.formula(formula))$term.labels
-    
+
     testing <- rep(0,3)
     for(i in 1:length(x)){
-      
+
       tmp <- strsplit(x[i],":")
       l <- length(tmp[[1]])
-      
+
       # interaction hypothesis of 2 factors
       if(l == 2){testing[3]<-1}
       else{
@@ -414,30 +425,30 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject, nonparametr
         else if(factor2 == x[i]){testing[2]<-1}
       }
     }
-    
+
     subplot<-colnames(dat2[,subplot])
     X<-data
     data <- colnames(dat)[1]
-    return(hrm.test.2.two(X, alpha , factor1, factor2, subject, data, formula, testing, nonparametric ))   
-    
+    return(hrm.test.2.two(X, alpha , factor1, factor2, subject, data, formula, testing, nonparametric ))
+
   }
-  
+
   # Case: no wholeplot, three subpot factor
   if(length(wholeplot) < 1 & length(subplot) == 3){
     factor1 <- colnames(dat2)[subplot[1]]
     factor2 <- colnames(dat2)[subplot[2]]
     factor3 <- colnames(dat2)[subplot[3]]
     x<-attributes(terms.formula(formula))$term.labels
-    
+
     testing <- rep(0,2^3-1)
     testing[1:3] <- 1
     for(i in 1:length(x)){
-      
+
       tmp <- strsplit(x[i],":")
       l <- length(tmp[[1]])
-      
+
       # interaction hypothesis of 2 factors
-      if(l == 2){ 
+      if(l == 2){
         if(grepl(factor1,x[i])){
           if(grepl(factor2,x[i])){
             testing[4] <- 1
@@ -448,21 +459,21 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject, nonparametr
           testing[6] <- 1
         }
       }
-      
+
       # interaction hypothesis of 3 factors
       if(l == 3){
         testing[7] <- 1
       }
     }
-    
+
     subplot<-colnames(dat2[,subplot])
     X<-data
     data <- colnames(dat)[1]
-    return(hrm.test.3.three(X, alpha , factor1, factor2, factor3, subject, data, formula, testing, nonparametric ))   
-    
+    return(hrm.test.3.three(X, alpha , factor1, factor2, factor3, subject, data, formula, testing, nonparametric ))
+
   }
-  
-  
+
+
   # Case: no wholeplot, four subpot factor
   if(length(wholeplot) < 1 & length(subplot) == 4){
     factor1 <- colnames(dat2)[subplot[1]]
@@ -473,8 +484,8 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject, nonparametr
     subplot<-colnames(dat2[,subplot])
     X<-data
     data <- colnames(dat)[1]
-    return(hrm.test.4.four(X, alpha , factor1, factor2, factor3, factor4, subject, data, formula, testing = rep(1,2^4-1), nonparametric ))   
-    
+    return(hrm.test.4.four(X, alpha , factor1, factor2, factor3, factor4, subject, data, formula, testing = rep(1,2^4-1), nonparametric ))
+
   }
 
   # Case: no wholeplot, five subpot factor
@@ -488,40 +499,40 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject, nonparametr
     subplot<-colnames(dat2[,subplot])
     X<-data
     data <- colnames(dat)[1]
-    return(hrm.test.5.five(X, alpha , factor1, factor2, factor3, factor4, factor5, subject, data, formula, testing = rep(1,2^5-1), nonparametric ))   
-    
-  }  
-  
+    return(hrm.test.5.five(X, alpha , factor1, factor2, factor3, factor4, factor5, subject, data, formula, testing = rep(1,2^5-1), nonparametric ))
+
+  }
+
   # Case: one wholeplot, no subpot factor
   if(length(wholeplot) == 1 & length(subplot) < 1){
     group <- colnames(dat2)[wholeplot[1]]
     x<-attributes(terms.formula(formula))$term.labels
-    
+
     wholeplot<-colnames(dat2[,group])
     X<-data
     data <- colnames(dat)[1]
-    return(hrm.test.1.none(X, alpha , group, subject, data, formula, nonparametric ))  
-  
+    return(hrm.test.1.none(X, alpha , group, subject, data, formula, nonparametric ))
+
   }
-  
+
   # Case: 1 whole and 1 subplot factor
   if(length(wholeplot)==1 & length(subplot)==1){
     group <- colnames(dat2)[wholeplot[1]]
     factor1 <- colnames(dat2)[subplot[1]]
     x<-attributes(terms.formula(formula))$term.labels
-    
+
     wholeplot<-colnames(dat2[,wholeplot])
     subplot<-colnames(dat2[,subplot])
-    
+
     testing <- rep(0,4)
     for(i in 1:length(x)){
-      
+
       tmp <- strsplit(x[i],":")
       l <- length(tmp[[1]])
-      
+
       # interaction hypothesis of 2 factors
       if(l == 2){testing[4]<-1}
-      
+
       else{
         if(group == x[i]){
           testing[1]<-1
@@ -532,28 +543,28 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject, nonparametr
     }
     X<-data
     data <- colnames(dat)[1]
-    return(hrm.test.2.one(X, alpha, group , factor1, subject, data, testing, formula, nonparametric ))   
+    return(hrm.test.2.one(X, alpha, group , factor1, subject, data, testing, formula, nonparametric ))
   }
-  
+
   # Case: 2 wholeplot, 1 subplot factor
   if(length(wholeplot)==2 & length(subplot)==1){
     group <- colnames(dat2)[wholeplot[1]]
     subgroup <- colnames(dat2)[wholeplot[2]]
     factor1 <- colnames(dat2)[subplot[1]]
     x<-attributes(terms.formula(formula))$term.labels
-    
+
     wholeplot<-colnames(dat2[,wholeplot])
     subplot<-colnames(dat2[,subplot])
-    
+
     testing <- rep(0,7)
     for(i in 1:length(x)){
-      
+
       tmp <- strsplit(x[i],":")
       l <- length(tmp[[1]])
-      
+
       # interaction hypothesis of 4 factors
       if(l == 3){testing[7]<-1}
-      
+
       # find out which interaction hypothesis of 2 factors is tested
       else if(l==2){
         if(grepl(group,x[i])){
@@ -579,28 +590,28 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject, nonparametr
     }
     X<-data
     data <- colnames(dat)[1]
-    return(hrm.test.2.between(X, alpha, group , subgroup, factor1, subject, data, testing, formula, nonparametric )) 
+    return(hrm.test.2.between(X, alpha, group , subgroup, factor1, subject, data, testing, formula, nonparametric ))
   }
-  
+
   # Case: 1 wholeplot, 2 subplot factors
   if(length(wholeplot)==1 & length(subplot)==2){
     group <- colnames(dat2)[wholeplot[1]]
     factor1 <- colnames(dat2)[subplot[1]]
     factor2 <- colnames(dat2)[subplot[2]]
     x<-attributes(terms.formula(formula))$term.labels
-    
+
     wholeplot<-colnames(dat2[,wholeplot])
     subplot<-colnames(dat2[,subplot])
-    
+
     testing <- rep(0,7)
     for(i in 1:length(x)){
-      
+
       tmp <- strsplit(x[i],":")
       l <- length(tmp[[1]])
-      
+
       # interaction hypothesis of 3 factors
       if(l == 3){testing[7]<-1}
-      
+
       # find out which interaction hypothesis of 2 factors is tested
       else if(l==2){
         if(grepl(group,x[i])){
@@ -626,28 +637,28 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject, nonparametr
     data <- colnames(dat)[1]
     return(hrm.test.2.within(X, alpha, group , factor1, factor2, subject, data, testing, formula, nonparametric ))
   }
-  
+
   # Case: 1 wholeplot, 3 subplot factors
   if(length(wholeplot)==1 & length(subplot)==3){
     group <- colnames(dat2)[wholeplot[1]]
     factor1 <- colnames(dat2)[subplot[1]]
     factor2 <- colnames(dat2)[subplot[2]]
     factor3 <- colnames(dat2)[subplot[3]]
-    
+
     x<-attributes(terms.formula(formula))$term.labels
-    
+
     wholeplot<-colnames(dat2[,wholeplot])
     subplot<-colnames(dat2[,subplot])
-    
+
     testing <- rep(0,15)
     for(i in 1:length(x)){
-      
+
       tmp <- strsplit(x[i],":")
       l <- length(tmp[[1]])
-      
+
       # interaction hypothesis of 3 factors
       if(l == 4){testing[15]<-1}
-      
+
       # find out which interaction hypothesis of 3 factors is tested
       else if(l==3){
         if(grepl(group,x[i])){
@@ -715,28 +726,28 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject, nonparametr
     data <- colnames(dat)[1]
     return(hrm.test.3.between(X, alpha, group , factor1, factor2, factor3, subject, data, testing, formula, nonparametric ))
   }
-  
-  
-  
+
+
+
   if(length(wholeplot)==2 & length(subplot)==2){
     group <- colnames(dat2)[wholeplot[1]]
     subgroup <- colnames(dat2)[wholeplot[2]]
     factor1 <- colnames(dat2)[subplot[1]]
     factor2 <- colnames(dat2)[subplot[2]]
     x<-attributes(terms.formula(formula))$term.labels
-    
+
     wholeplot<-colnames(dat2[,wholeplot])
     subplot<-colnames(dat2[,subplot])
-    
+
     testing <- rep(0,15)
     for(i in 1:length(x)){
-      
+
       tmp <- strsplit(x[i],":")
       l <- length(tmp[[1]])
-      
+
       # interaction hypothesis of 4 factors
       if(l == 4){testing[15]<-1}
-      
+
       # find out which interaction hypothesis of 3 factors is tested
       else if(l==3){
         if(grepl(group,x[i])){
@@ -756,7 +767,7 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject, nonparametr
           testing[14]<-1
         }
       }
-      
+
       # find out which interaction hypothesis of 2 factors is tested
       else if(l==2){
         if(grepl(group,x[i])){
@@ -796,4 +807,4 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject, nonparametr
     data <- colnames(dat)[1]
     return(hrm.test.2.between.within(X, alpha, group , subgroup, factor1, factor2, subject, data, testing, formula, nonparametric ))
   }
-}
\ No newline at end of file
+}
