@@ -2,13 +2,13 @@
 ### Filename:    S3methods.R
 ### Description: S3 Generic Methods for calculating the test statistic; Providing different ways to to 'input' the data
 ###              and methods for print/summary are defined here
-###              
+###
 ###
 ###
 ####################################################################################################################################
 
-#' Test for Multi-Factor High-Dimensional Repeated Measures 
-#' 
+#' Test for Multi-Factor High-Dimensional Repeated Measures
+#'
 #' @description Performing main and interaction effects of up to three whole- or subplot-factors. In total, a maximum of four factors can be used. There are two different S3 methods available. The first method requires a list of matrices in the wide table format. The second methodl requres a data.frame in the long table format.
 #' @rdname hrm_test
 #' @param data Either a data.frame (one observation per row) or a list with matrices (one subject per row) for all groups containing the data
@@ -16,6 +16,7 @@
 #' @param subject column name within the data frame X identifying the subjects
 #' @param alpha alpha level used for calculating the critical value for the test
 #' @param nonparametric Logical variable indicating wether the noparametric version of the test statistic should be used
+#' @param character.only a logical indicating whether subject can be assumed to be a character string
 #' @param ... Further arguments passed to 'hrm_test' will be ignored
 #' @return Returns an object from class HRM containing
 #' @return \item{result}{A dataframe with the results from the hypotheses tests.}
@@ -46,7 +47,12 @@ hrm_test.list <- function(data, alpha = 0.05, ...) {
 #' @rdname hrm_test
 #' @method hrm_test data.frame
 #' @keywords export
-hrm_test.data.frame <- function(data, formula, alpha = 0.05,  subject, nonparametric = FALSE, ... ) { 
+hrm_test.data.frame <- function(data, formula, alpha = 0.05,  subject, nonparametric = FALSE,
+                                character.only = FALSE, ... ) {
+  if(!character.only) {
+    subject <- as.character(substitute(subject))
+  }
+
   return(hrm_test_internal(formula=formula, data=data, alpha=alpha,subject=subject, nonparametric ))
 }
 
@@ -62,7 +68,7 @@ print.HRM <- function(x, ...) {
     print(x$formula)
     cat("\n")
   }
-  
+
   print(x$result, row.names = FALSE)
   cat("\nSignif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1")
   cat("\n")
@@ -88,8 +94,8 @@ summary.HRM <- function(object, ...) {
     cat("\n")
     cat("\n")
   }
-  
+
   print(object$result, row.names = FALSE)
   cat("\nSignif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1")
   cat("\n")
-}
\ No newline at end of file
+}
